#include "pcapwrapper.h"

pcap_t * _handle;

void terminate_process (int) {
	pcap_breakloop (_handle);
}

void got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet) {
	time_t nowtime;
	struct tm *nowtm;
	char tmbuf [64];

	nowtime = header->ts.tv_sec;
	nowtm = localtime(&nowtime);
	strftime(tmbuf, sizeof tmbuf, "%Y-%m-%d %T", nowtm);

	cout << "[" << tmbuf << "." << header->ts.tv_usec << "]" << " Got packet of " << header->caplen << " bytes" << endl;
}


PcapWrapper::PcapWrapper (char * filter_) {

	dev = pcap_lookupdev (errbuf);

	if (dev == NULL) {
		cerr << "Couldn't find default device: " << errbuf << endl;
		cerr << "*** You must probably run as root! ***" << endl;
		exit (EXIT_FAILURE);
	}

	cout << "Device: " << dev << endl;

	// Find the properties for the device 
	if (pcap_lookupnet (dev, &net, &mask, errbuf) == -1) {
		cerr << "Couldn't get netmask for device" << dev << ":" << errbuf;
		net = 0;
		mask = 0;
	}

	handle = pcap_open_live (dev, BUFSIZ, 1, 0, errbuf);
	if (handle == NULL) {
		cerr << "Couldn't open device " << dev << " : " << errbuf << endl;
		exit (EXIT_FAILURE);
	}

	if (filter_ != NULL)
		setFilter (filter);
}

PcapWrapper::~PcapWrapper () {
	cout << "\nFinishing the capture... that's all folks!" << endl;

	if (filter != NULL) pcap_freecode(&fp); 
	pcap_close (handle);
}

void PcapWrapper::run (int stoptime_) {
	// stop after stoptime seconds
	if (stoptime_ > 0) {
		cout << "*** Finishing after " << stoptime_ << " seconds ***" << endl;
		signal (SIGALRM, terminate_process);
		alarm (stoptime_);
		_handle = handle;
	} else {
		cout << "*** Press ctrl-c to finish ***" << endl;
		signal (SIGINT, terminate_process);
		_handle = handle;
	}

	// now we can set our callback function 
	pcap_loop (handle, -1, got_packet, NULL);
}

void PcapWrapper::setFilter (char * filter_) {
	filter = filter_;
	
	// Compile and apply the filter 
	if (pcap_compile (handle, &fp, filter_, 0, net) == -1) {
		cerr << "Couldn't parse filter " << filter_ << ":" << pcap_geterr (handle);
		exit (EXIT_FAILURE);
	}
	
	if (pcap_setfilter (handle, &fp) == -1) {
		cerr << "Couldn't parse filter " << filter_ << ":" << pcap_geterr (handle);
		exit (EXIT_FAILURE);
	}
}

